'************************************************
'*  PAL/NTSC Video Driver - 24bit SDRAM Driver  *
'************************************************

CON
  _clkmode = xinput
  _xinfreq = 60_000_000

CLOCK_FREQ      = 60_000_000

SERIAL_TX       = 90
SERIAL_RX       = 91

COMMANDBUFFER   = $1FF80
VIDEOSCANLINE   = $1FFF4
BORDERCOLOUR    = $1FFF8
VIDEOSYNC       = $1FFFC
SERIALBUFFER    = $1F800
BITMAPBUFFER    = $1e000


SETPAL		= 0
SETINTERLACED	= 1

s		= 46            'scales DAC output (s = 0..128)  (adjust slightly to optimize slight color artifacts)
r		= s * 78 / 128  'adjusts for modulator expansion
lntsc		= 3640          'NTSC colorburst cycles per line * 16
sntsc		= 624           'NTSC colorburst cycles per sync * 16
lpal		= 4540          'PAL colorburst cycles per line * 16
spal		= 854           'PAL colorburst cycles per sync * 16

FRAME_XSIZE	= 512
FRAME_YSIZE	= 480/(SETINTERLACED+1)

SYNC_SIZE	= 15
PAL_FULL	= 625-15
PAL_BF		= ((((PAL_FULL/2)-FRAME_YSIZE)-17)/2)
PAL_AF		= (((PAL_FULL/2)-FRAME_YSIZE)-17)-PAL_BF
PAL_SCALER	= 6	'make this 5*2 if doing 320 or less XSIZE otherwise 5

NTSC_FULL	= 525-15
NTSC_BF		= ((((NTSC_FULL/2)-FRAME_YSIZE)-17)/2)
NTSC_AF		= (((NTSC_FULL/2)-FRAME_YSIZE)-17)-NTSC_BF
NTSC_SCALER	= 4	'make this 4*2 if doing 320 or less XSIZE otherwise 4

'-------------------------

DAT		org

		reps	#4*2,#1
		setptra	hubram_loc              'write initial command list
		wrlong	h00000000,ptra++        '4 commands for SDRAM 2 longs each command
		wrlong	h00000008,ptra++        'terminate with end_of_list

		setctra	#%00111                 'set ctra for PLL*8
		setvid	#%1110                  'set vid for composite output

		cfgdac0	#%11                    'set bottom dac to video mode (YIQ)
		cfgpins	pinmask,pindacs         'set P0 for dac output

		setcog  #8                      'launch SDRAM driver into cog1
		coginit sdram_adr,hubram_loc
		coginit serial_adr,serial_par

		mov	palsw,#0

		movs	:mod,#nfrqa_
		cmp	doingpal,#1 wz
	if_z	movs	:mod,#pfrqa_
		nop
		nop
		reps	#20,#1
:mod		setinds	#frqa_,#nfrqa_
		mov	indb++,inda++
                setfrqa frqa_                   'set frqa
                setvidy vidy_                   'set yiq coefficients
                setvidi vidi_
		setvidq vidq_                   'pos Q modulation

'------------------------------
' Field loop
:lp	        mov     tv_x,#5                    'high vertical syncs
                call    #vsynch                 'non interlaced PAL (312 lines)
                mov     tv_x,#5                    'low vertical syncs
                call    #vsyncl
                mov     tv_x,#4                    'high vertical syncs
                call    #vsynch
		xor	palsw,#1

                mov     sdram_ptr,sdram_ptr_start
		mov	doingfield,#0
		mov	doingline,#0
		call	#field

		cmp	doinginterlaced,#0 wz
	if_z	jmp	#:lp

	        mov     tv_x,#6                    'high vertical syncs
                call    #vsynch                 'non interlaced PAL (312 lines)
                mov     tv_x,#5                    'low vertical syncs
                call    #vsyncl
                mov     tv_x,#5                    'high vertical syncs
                call    #vsynch

                mov     sdram_ptr,sdram_ptr_start
		add	sdram_ptr,oneline
		mov	doingfield,#1
		mov	doingline,#1
		call	#field

		jmp	#:lp

'------------------------------------

field		mov     tv_x,v_bf                  'top blanks
		cmp	doingpal,#1 wz
	if_z	add	tv_x,#15-1
	if_nz	add	tv_x,#8-1
		call    #blank

		wrlong	sdram_ptr,sdram_loc
		add	sdram_ptr,sdram_add
		wrlong	hubram_rd1,hubram_loc
		wrlong	sdram_ptr,sdram_loc2
		add	sdram_ptr,sdram_add
		wrlong	hubram_rd2,hubram_loc2
                add	sdram_ptr,sdram_add

		mov	tv_x,#1
		call    #blank

		wrlong	doingline,tv_lineptr
		add	doingline,#1

                wrlong	sdram_ptr,sdram_loc
                add	sdram_ptr,sdram_add
                wrlong	hubram_rd3,hubram_loc
                wrlong	sdram_ptr,sdram_loc2
                add	sdram_ptr,sdram_add
                wrlong	hubram_rd4,hubram_loc2
                add	sdram_ptr,sdram_add

		mov     tv_y,#FRAME_YSIZE           'set visible lines
line		call    #hsync                  'do horizontal sync
		waitvid v_bo,#0                 'black left border

		wrlong	doingline,tv_lineptr
		add	doingline,#1

                wrlong  sdram_ptr,sdram_loc
                add     sdram_ptr,sdram_add
                wrlong  hubram_rd5,hubram_loc
                wrlong  sdram_ptr,sdram_loc2
                add     sdram_ptr,sdram_add
                wrlong  hubram_rd6,hubram_loc2
                add	sdram_ptr,sdram_add

                mov     tvtmp,xpixel
                shl     tvtmp,#2
                add     tvtmp,hubram_ptr1
                setptra tvtmp

                mov     tvtmp,hubram_rd1
                mov     hubram_rd1,hubram_rd3
                mov     hubram_rd3,hubram_rd5
                mov     hubram_rd5,tvtmp
                mov     tvtmp,hubram_rd2
                mov     hubram_rd2,hubram_rd4
                mov     hubram_rd4,hubram_rd6
                mov     hubram_rd6,tvtmp
                mov     tvtmp,hubram_ptr1
                mov     hubram_ptr1,hubram_ptr2
                mov     hubram_ptr2,hubram_ptr3
                mov     hubram_ptr3,tvtmp


		mov	subtvx,#FRAME_XSIZE/64
:lp1            waitvid mode4,#$00              '64 pixels
                notb    :lp1,#6
                notb    :mod1,#9+6
                reps    #64,#2
:mod1           setspa  #$00
                rdlongc tvtmp,ptra++
                pusha   tvtmp
                djnz    subtvx,#:lp1

		cmp	doinginterlaced,#1 wz
	if_z	add	sdram_ptr,oneline

		waitvid v_bo,#0                 'black right border
		djnz    tv_y,#line                 'another line?

		wrlong  one,tv_VSyncPtr

		mov     tv_x,v_af                   'bottom blanks
		cmp	doingpal,#1 wz
	if_z	add	tv_x,#2
	if_nz	add	tv_x,#9
		call    #blank

field_ret	ret

blank		call    #hsync                  'blank lines
		waitvid v_bl,#0
		djnz    tv_x,#blank
blank_ret	ret

hsync		waitvid v_bs,c_sh               'horizontal sync
		waitvid v_sn,c_sl
		waitvid v_bc,c_sh
		xor     palsw,#1  wz            'PAL switch
	if_nz	setvidq vidq_                   'pos Q modulation
	if_z	setvidq vidqn_                  'neg Q modulation
		waitvid v_cb,c_cb
		waitvid v_bv,c_sh
hsync_ret	ret

vsynch		waitvid v_bs,c_sh               'vertical sync high
		waitvid v_hs,c_sl
		waitvid v_hr,c_sh
		djnz    tv_x,#vsynch
vsynch_ret	ret

vsyncl		waitvid v_bs,c_sh               'vertical sync low
		waitvid v_ls,c_sl
		waitvid v_lr,c_sh
		djnz    tv_x,#vsyncl
vsyncl_ret	ret

'
' Initialized data - video
'
pinmask	long	%1                      'pin mask for P0
pindacs	long	%1111_101100000         'pin configuration for video DAC

                '       color base      mode            clocks/pixel    clocks/waitvid

doingpal	long	SETPAL
doinginterlaced	long	SETINTERLACED
doingline	long	0
tv_lineptr	long	VIDEOSCANLINE

tvand		long	$00ffffff

xpixel		long	0
v_xsize		long	FRAME_XSIZE
c_sh            long    $1_000000               'sync high
c_sl            long    $2_000000               'sync low

'NTSC Data
nfrqa_          long    $1E8B_A2A7              '57.272720 MHz (colorburst * 16)
nvidy_          long    (110*s/128 & $FF) << 24 + ((+38*s/128) & $FF) << 16 + ((+75*s/128) & $FF) << 8 + ((+15*s/128) & $FF)
nvidi_          long    (100*s/128 & $FF) << 24 + ((+76*r/128) & $FF) << 16 + ((-35*r/128) & $FF) << 8 + ((-41*r/128) & $FF)
nvidq_          long                  128 << 24 + ((+27*r/128) & $FF) << 16 + ((-67*r/128) & $FF) << 8 + ((+40*r/128) & $FF)
nvidqn_         long                  128 << 24 + ((+27*r/128) & $FF) << 16 + ((-67*r/128) & $FF) << 8 + ((+40*r/128) & $FF)
                '       color base      mode            clocks/pixel    clocks/waitvid
nmode4          long    $00 << 24 +     $f << 20 +      NTSC_SCALER << 13 +       64*NTSC_SCALER
nv_bo           long    (lntsc-sntsc-FRAME_XSIZE*NTSC_SCALER)/2   'left/right border
nv_bs           long    100                     'before sync            horizontal sync
nv_sn           long    269                     'sync
nv_bc           long    50                      'before colorburst
nv_cb           long    144                     'colorburst
nv_bv           long    sntsc-144-50-269-100    'before visible
nv_hs           long    135                     'high sync              high vertical sync
nv_hr           long    lntsc/2-135-100         'high sync remainder
nv_ls           long    1564                    'low sync               low vertical sync
nv_lr           long    lntsc/2-1564-100        'low sync remainder
nv_bl           long    lntsc-sntsc             'blank visible
nc_cb           long    $1_687800               'colorburst reference color
nv_bf		long	NTSC_BF			'blanks before frame
nv_af		long	NTSC_AF			'blanks after frame

'PAL Data
pfrqa_          long    $25D5_6350 +0           '70.9379 MHz (4.43361875 * 16) (+/-Tune value)
pvidy_          long    (110*s/128 & $FF) << 24 + ((+38*s/128) & $FF) << 16 + ((+75*s/128) & $FF) << 8 + ((+15*s/128) & $FF)
pvidi_          long    (100*s/128 & $FF) << 24 + ((-19*r/128) & $FF) << 16 + ((-37*r/128) & $FF) << 8 + ((+55*r/128) & $FF)
pvidq_          long                  128 << 24 + ((+78*r/128) & $FF) << 16 + ((-65*r/128) & $FF) << 8 + ((-13*r/128) & $FF)
pvidqn_         long                  128 << 24 + ((-78*r/128) & $FF) << 16 + ((+65*r/128) & $FF) << 8 + ((+13*r/128) & $FF)
                '       color base      mode            clocks/pixel    clocks/waitvid
pmode4          long    $00 << 24 +     $f << 20 +      PAL_SCALER << 13 +       64*PAL_SCALER
pv_bo           long    (lpal-spal-FRAME_XSIZE*PAL_SCALER)/2    'left/right border
pv_bs           long    117                     'before sync            horizontal sync
pv_sn           long    333                     'sync
pv_bc           long    63                      'before colorburst
pv_cb           long    160                     'colorburst (10)
pv_bv           long    spal-160-63-333-117     'before visible
pv_hs           long    167                     'high sync              high vertical sync
pv_hr           long    lpal/2-167-117          'high sync remainder
pv_ls           long    1937                    'low sync               low vertical sync
pv_lr           long    lpal/2-1937-117         'low sync remainder
pv_bl           long    lpal-spal               'blank visible
pc_cb           long    $1_905000               'colorburst reference color PAL (orange)
pv_bf		long	PAL_BF			'blanks before frame
pv_af		long	PAL_AF			'blanks after frame

one		long	1

tv_VSyncPtr	long	VIDEOSYNC

serial_adr      long    @Serial_Driver+$e80
serial_par      long    90<<9+91

sdram_adr       long    @SDRAM_Driver+$e80
hubram_loc      long    COMMANDBUFFER
sdram_loc       long    COMMANDBUFFER+4
hubram_loc2     long    COMMANDBUFFER+8
sdram_loc2      long    COMMANDBUFFER+12
sdram_add       long    256*4
oneline         long    768*4

h00000000       long    $00000000
h00000008       long    $00000008

hubram_rd1      long    BITMAPBUFFER+7+$0000
hubram_rd2      long    BITMAPBUFFER+7+$0400
hubram_rd3      long    BITMAPBUFFER+7+$0800
hubram_rd4      long    BITMAPBUFFER+7+$0c00
hubram_rd5      long    BITMAPBUFFER+7+$1000
hubram_rd6      long    BITMAPBUFFER+7+$1400
hubram_ptr1     long    BITMAPBUFFER+$0000
hubram_ptr2     long    BITMAPBUFFER+$0800
hubram_ptr3     long    BITMAPBUFFER+$1000

sdram_ptr_start long    0

sdram_ptr       res     1
doingfield	res	1

subtvx		res	1

tv_x		res	1
tv_y		res	1
tvtmp		res	1
palsw		res	1

frqa_           res	1	'long    $1E8B_A2A7              '57.272720 MHz (colorburst * 16)
vidy_           res	1	'long    (110*s/128 & $FF) << 24 + ((+38*s/128) & $FF) << 16 + ((+75*s/128) & $FF) << 8 + ((+15*s/128) & $FF)
vidi_           res	1	'long    (100*s/128 & $FF) << 24 + ((+76*r/128) & $FF) << 16 + ((-35*r/128) & $FF) << 8 + ((-41*r/128) & $FF)
vidq_           res	1	'long                  128 << 24 + ((+27*r/128) & $FF) << 16 + ((-67*r/128) & $FF) << 8 + ((+40*r/128) & $FF)
vidqn_          res	1	'long                  128 << 24 + ((+27*r/128) & $FF) << 16 + ((-67*r/128) & $FF) << 8 + ((+40*r/128) & $FF)
                '       color base      mode            clocks/pixel    clocks/waitvid
mode4           res	1	'long    $00 << 24 +     $4 << 20 +      4 << 13 +       4*4
v_bo            res	1	'long    (lntsc-sntsc-FRAME_XSIZE*4)/2   'left/right border
v_bs            res	1	'long    100                     'before sync            horizontal sync
v_sn            res	1	'long    269                     'sync
v_bc            res	1	'long    50                      'before colorburst
v_cb            res	1	'long    144                     'colorburst
v_bv            res	1	'long    sntsc-144-50-269-100    'before visible
v_hs            res	1	'long    135                     'high sync              high vertical sync
v_hr            res	1	'long    lntsc/2-135-100         'high sync remainder
v_ls            res	1	'long    1564                    'low sync               low vertical sync
v_lr            res	1	'long    lntsc/2-1564-100        'low sync remainder
v_bl            res	1	'long    lntsc-sntsc             'blank visible
c_cb            res	1	'long    $1_687800               'colorburst reference color
v_bf		res	1	'long	NTSC_BF			'blanks before frame
v_af		res	1	'long	NTSC_AF			'blanks after frame

'------------------
'-------------------------------------------------------------

' SDRAM Driver Demo
'
'       - starts SDRAM driver in cog1'       - starts Monitor in cog0
'
DAT             org     0
driver                                          'SDRAM driver follows
'********************************
'*                              *
'*  Propeller II SDRAM Driver   *
'*     for 16M x 16 devices     *
'*        (FPGA version)        *
'*                              *
'*  Version 0.1                 *
'*  9 April 2013                *
'*  by Chip Gracey              *
'*                              *
'********************************

{
SDRAM connections:

        P85 = cke (held high)           Port C
        P84 = cs
        P83 = ras
        P82 = cas
        P81 = we
        P80 = udqm (not used)
        P79 = ldqm (not used)
   P78..P77 = ba[1..0]
   P76..P64 = a[12..0]

   P63..P48 = dq[15..0]                 Port B


  Note: All pin directions have a 2-clock delay
        All pin outputs going to SDRAM have a 3-clock delay, since they are registered at the pin
        All pin inputs coming from SDRAM have a 2-clock delay, since they are registered at the pin


Commands (pairs of longs):

name            quads   bytes   hub_address (+0, set 2nd)                       sdram_address (+1, set 1st)
------------------------------------------------------------------------------------------------------------------------
rw_1024         64      1024    %xxxx_xxxx_xxxx_xxxA_AAAA_AAAA_AAAA_W111        %xxxx_xxxA_AAAA_AAAA_AAAA_AA00_0000_0000
rw_512          32       512    %xxxx_xxxx_xxxx_xxxA_AAAA_AAAA_AAAA_W110        %xxxx_xxxA_AAAA_AAAA_AAAA_AAA0_0000_0000
rw_256          16       256    %xxxx_xxxx_xxxx_xxxA_AAAA_AAAA_AAAA_W101        %xxxx_xxxA_AAAA_AAAA_AAAA_AAAA_0000_0000
rw_128           8       128    %xxxx_xxxx_xxxx_xxxA_AAAA_AAAA_AAAA_W100        %xxxx_xxxA_AAAA_AAAA_AAAA_AAAA_A000_0000
rw_64            4        64    %xxxx_xxxx_xxxx_xxxA_AAAA_AAAA_AAAA_W011        %xxxx_xxxA_AAAA_AAAA_AAAA_AAAA_AA00_0000
rw_32            2        32    %xxxx_xxxx_xxxx_xxxA_AAAA_AAAA_AAAA_W010        %xxxx_xxxA_AAAA_AAAA_AAAA_AAAA_AAA0_0000
rw_16            1        16    %xxxx_xxxx_xxxx_xxxA_AAAA_AAAA_AAAA_W001        %xxxx_xxxA_AAAA_AAAA_AAAA_AAAA_AAAA_0000
skip_done        0         0    %0000_0000_0000_0000_0000_0000_0000_0000        %xxxx_xxxx_xxxx_xxxx_xxxx_xxxx_xxxx_xxxx
end_of_list      0         0    %0000_0000_0000_0000_0000_0000_0000_1000        %xxxx_xxxx_xxxx_xxxx_xxxx_xxxx_xxxx_xxxx


The driver scans a list for commands. Commands take two longs and are structured as hub_address + sdram_address. To have
the driver perform a read or write operation on the SDRAM, first set the sdram_address (2nd long) and then the hub_address
(1st long). The driver will set each hub_address long to skip_done (0) when its associated operation has completed.

Before starting the driver, build the command list structure with pairs of 0's, then terminate it with an 8. When launching
the driver into a cog, point the parameter (S) to the start of the command list.


Command list (longs):

        hub_address (skip_done/rw_xxxx)
        sdram_address

        hub_address (skip_done/rw_xxxx)
        sdram_address

        hub_address (skip_done/rw_xxxx)
        sdram_address

        ...

        end_of_list
}

DAT             org
'
'
' Initialize SDRAM                              clocks  hub     
'                                               ----------------
sdram_driver    getptra list                    '1      -               save command list address

                reps    #6000,#1                '1      -               repeat instruction for 100us @60MHz
                mov     pinc,h003FFFFF          '1      -               cke and cs high
                mov     dirc,h003FFFFF  wz      '6000   -               set SDRAM signals to outputs (100us), Z = 0
'
'
' Scan list for commands
'
:finish if_z    wrlong  cmd,ptra[-2]            '1      0               if command finished, set it to skip_done (cmd = 0)

                mov     pinc,h00240400          '1      1               issue 'precharge all' command
                setb    pinc,#20        wz      '1      2               deselect, Z = 0
:reset  if_z    setptra list                    '1      3               if end_of_list, reset list pointer; satisy Trp

                mov     pinc,h00200037          '1      4               issue 'set mode' command
                setb    pinc,#20                '1      5               deselect, satisfy Tmrd

                mov     pinc,h00226000          '1      6               issue 'auto refresh' command
                setb    pinc,#20                '1      7               deselect, satisfy Trfc in 8 more clocks

:next           rdlong  cmd,ptra++[2]   wz      '3      0..2            check command, point to next
                setptrb cmd                     '1      3               in case command, set hub address
        if_z    jmp     #:next                  '1/4    4/4..7          if skip_done, next command

                decod3  cmd             wc      '1      5               decode size bits, C = write
                and     cmd,#%11111110  wz      '1      6               isolate bits 7..1, Z = 1 if end_of_list
        if_z    jmp     #:reset                 '1/4    7/7..2          if end_of_list, reset list pointer
'
'
' Execute read/write command                    clocks  hub     XFR write SDRAM         XFR read SDRAM
'                                               ------------------------------------------------------
                rdlong  adr,ptra[-1]            '3      0..2    -                       -                       get SDRAM address

                shl     adr,#7                  '1      3       -                       -                       make 'active' command with bank and row address
                or      adr,#%1_0011_00         '1      4       -                       -                       (ba[1..0], a[12..0]) = adr[24..10]
                rol     adr,#15                 '1      5       -                       -

                mov     pinc,adr                '1      6       -                       -                       issue 'active' command
                setb    pinc,#20                '1      7       -                       -                       deselect, satisfy Trcd in 1 more clock

        if_c    rdquad  ptrb++                  '1      0       -                       -                       if write, read initial QUADs from hub

                setbc   :quad,#0                '1      1       -                       -                       set wrquad/rdquad according to read/write

        if_c    setxfr  #%010_011               '1      2       <QUADs_to_16_pins>      -                       if write, configure XFR at hub cycle 2

                shr     adr,#22+1               '1      3       -                       -                       make blank command with bank and column address
                and     pinc,h00306000          '1      4       -                       -                       (ba[1..0], a[12..0]) = (adr[24..23], %0000, adr[9..1])
                or      pinc,adr                '1      5       -                       -

        if_c    mov     dirb,hFFFF0000          '1      6       -                       -                       if write, enable data outputs

        if_nc   setxfr  #%100_011               '1      7                               <16_pins_to_QUADs>      if read, configure XFR at hub cycle 7

        if_nc   xor     pinc,h001A0000          '1      0       -                       -                       if read, issue 'read' command
        if_nc   setb    pinc,#20                '1      1       -                       -                       if read, deselect

        if_c    xor     pinc,h00180000          '1      2       -                       -                       if write, issue 'write' command (aligns with XFR on next clock)
        if_c    setb    pinc,#20                '1      3       output QUAD0 w0         -                       if write, deselect; read: SDRAM sees 'read' command

                shr     cmd,#1                  '1      4       output QUAD0 w1         -                       get loop count

        if_nc   nop     #7                      '1      5       output QUAD1 w0         -                       if read, pad time
                '(nop)                          '1      6                               -                       read: SDRAM starts outputting data stream
                '(nop)                          '1      7                               -
                '(nop)                          '1      0                               input w0                read: SDRAM data stream begins arriving in XFR
                '(nop)                          '1      1                               input w1 -> QUAD0
                '(nop)                          '1      2                               input w0
                '(nop)                          '1      3                               input w1 -> QUAD1
                '(nop)                          '1      4                               input w0

:quad           '(wrquad/rdquad)                '1      5       output QUAD1 w0         input w1 -> QUAD2
                '(wrquad/rdquad)                '1      6       output QUAD1 w1         input w0
                '(wrquad/rdquad)                '1      7       output QUAD2 w0         input w1 -> QUAD3
                wrquad  ptrb++                  '1      0       output QUAD2 w1         input w0                write: read next QUADS; read: write current QUADS
                djnz    cmd,#:quad              '1      1       output QUAD3 w0         input w1 -> QUAD0       loop for each set of QUADs
                '(djnz looping)                 '1      2       output QUAD3 w1         input w0
                '(djnz looping)                 '1      3       output QUAD0 w0 (new)   input w1 -> QUAD1       write: RDQUAD data valid
                '(djnz looping)                 '1      4       output QUAD0 w1         input w0

                mov     pinc,h002C0000          '1      2       output QUAD3 w1         -                       issue 'burst terminate' command (aligns with XFR on next clock)
                mov     dirb,#0         wz      '1      3       -                       -                       cancel data outputs (aligns with 'burst terminate'), Z=1
                jmp     #:finish                '4      4..7    -                       -                       finish up, scan for next command
'
' Constants
'
h00180000       long    $00180000               'write' command toggle
h001A0000       long    $001A0000               'read' command toggle
h00200037       long    $00200037               'set mode' command (full-page r/w bursts, cas latency = 3)
h00226000       long    $00226000               'read' command mask
h00240400       long    $00240400               'precharge all' command
h002C0000       long    $002C0000               'burst terminate' command
h00306000       long    $00306000               'blank command mask
h003FFFFF       long    $003FFFFF               'control pins mask
hFFFF0000       long    $FFFF0000               'data pins mask
'
'
' Variables
'
list            res     1                       'command list pointer
cmd             res     1                       'command
adr             res     1                       'address

'--------------------------

        org     0
Serial_Driver
                setp    #SERIAL_TX
                setspa  #0
                setspb  #0

                mov     period,#0

                jmptask #baud_task,#%0010       'enable baud detector task
                settask #%%0101
                tjz     period,#$               'wait for <space> to set period

                jmptask #rxtask,#%0100
                settask #%%2021

:lp             call    #rx
                cmp     rxx,#"J" wz
        if_nz   jmp     #:lp
                call    #rx
                cmp     rxx,#"B" wz
        if_nz   jmp     #:lp

                call    #getlong
                mov     toadd,longval
                call    #getlong
                mov     tolen,longval

                call    #rx
                cmp     rxx,#"J" wz
        if_nz   jmp     #:lp
                call    #rx
                cmp     rxx,#"B" wz
        if_nz   jmp     #:lp

                call    #getbuf

                jmp     #:lp

data            long    $ffff
mytmp           long    0

getbuf          tjz     tolen,#getbuf_ret
                mov     rxsize,tolen
                cmp     rxsize,oneK wc,wz
        if_a    mov     rxsize,oneK
                mov     rxsiz,rxsize
                setptra rxbuffer
:lp             call    #rx
                wrbyte  rxx,ptra++
                add     brd,#1
                wrlong  brd,brdrptr
                djnz    rxsize,#:lp
                wrlong  toadd,rx_sdram_loc
                mov     rxx,rxbuffer
                or      rxx,#15 'write 1024 bytes
                wrlong  rxx,rx_hubram_loc
                add     toadd,rxsiz
                rdlong  rxx,rx_hubram_loc
                tjnz    rxx,#$-1
                sub     tolen,rxsiz wc,wz
        if_nc   jmp     #getbuf
getbuf_ret      ret

oneK            long    1024
rxsize          long    0
rxsiz           long    0
rx_hubram_loc   long    COMMANDBUFFER+16
rx_sdram_loc    long    COMMANDBUFFER+20
rxbuffer        long    SERIALBUFFER

getlong         call    #rx
                mov     longval,rxx
                call    #rx
                shl     rxx,#8
                or      longval,rxx
                call    #rx
                shl     rxx,#16
                or      longval,rxx
                call    #rx
                shl     rxx,#24
                or      longval,rxx
getlong_ret     ret
longval         long    0
toadd           long    0
tolen           long    0

rxtask          mov     rxbits,#9
                mov     rxdata,period
                sar     rxdata,#1
:rxwait         jp      rxpin,#:rxwait
                getcnt  rxtime
                add     rxtime,rxdata
                mov     rxdata,#0
:rxbit          rcr     rxdata,#1
                add     rxtime,period
                passcnt rxtime
                getp    rxpin           wc
                djnz    rxbits,#:rxbit
                shr     rxdata,#32-8
                pushb   rxdata
                jmp     #rxtask

rx              call    #rxcheck
        if_z    jmp     #rx
rx_ret          ret

rxcheck         getspb  rxhead
                getspa  rxtail
                cmp     rxhead,rxtail wz
        if_nz   popar   rxx
                and     rxx,#255
rxcheck_ret     ret

tx              shl     txx,#1
                setb    txx,#9
                getcnt  txw
:lp             add     txw,period
                passcnt txw
                shr     txx,#1 wc
                setpc   txpin
                tjnz    txx,#:lp
                setp    txpin
tx_ret          ret

'**********************
'* Baud Detector Task *
'**********************

baud_task       movd    ctr,rxpin              'set ctra to time rx pin states

:loop           notb    ctr,#5          wc      'if 1,0 sample set, c=0
                setctra ctr                     '($20 -> 10000001001 -> 1, 6x 0, 1x 1, 2x 0, 1)

        if_nc   mov     limh,buff0              'if 1,0 sample set,
        if_nc   shr     limh,#4                 '..make window from 1st 0 (6x if $20)
        if_nc   neg     liml,limh
        if_nc   add     limh,buff0
        if_nc   add     liml,buff0

        if_nc   mov     comp,buff1              'if 1,0 sample set,
        if_nc   mul     comp,#6                 '..normalize 2nd 1 (1x if $20) to 6x
        if_nc   cmpr    comp,limh       wc      '..check if within window
        if_nc   cmp     comp,liml       wc

        if_nc   mov     comp,buff2              'if 1,0 sample set,
        if_nc   mul     comp,#3                 '..normalize 2nd 0 (2x if $20) to 6x
        if_nc   cmpr    comp,limh       wc      '..check if within window
        if_nc   cmp     comp,liml       wc

        if_nc   add     buff0,buff2             'if $20,
        if_nc   shr     buff0,#3                '..compute period from 6x 0 and 2x 0
        if_nc   mov     period,buff0            '..update period

                mov     buff0,buff1             'scroll sample buffer
                mov     buff1,buff2

:wait           getcosa buff2                   'wait for next sample
                tjnz    buff2,#:loop
                jmp     #:wait

ctr             long    %100_01001              'ctr configuration for timing low on rx pin


txpin           long    SERIAL_TX
txx             long    0
txw             long    0
rxpin           long    SERIAL_RX
rxhead          long    0
rxtail          long    0
rxtime          long    0
rxdata          long    0
rxbits          long    0
rxx             long    0

mytask          long    %%0111_0111_0111_0111

brdrptr         long    BORDERCOLOUR
brd             long    0

buff0           res     1                       'baud detector task
buff1           res     1
buff2           res     1
limh            res     1
liml            res     1
comp            res     1
period          res     1

'----------------
